#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

# shellcheck disable=2154

chrun()
{
    # run a shell command in the chroot dir $MNT
    arch-chroot $MNT /bin/bash -c "$1"
}

json()
{
    # get a value from http://api.ipstack.com in json format using my API key
    # this includes: ip, geolocation, country name
    curl -s "http://api.ipstack.com/$2" | python3 -c "import sys, json; print(json.load(sys.stdin)['$1'])"
}

src()
{
    # source a file ($1), if it fails we die with an error message
    . "$1" || { printf "\nFailed to source file %s\n" "$1"; die 1; }
}

ssd()
{
    # returns 0 (true) when the device passed ($1) is NOT a rotational device
    local dev=$1
    dev=${dev#/dev/}
    [[ $dev =~ nvme ]] && dev=${dev%p[0-9]*} || dev=${dev%[0-9]*}
    [[ $(cat /sys/block/$dev/queue/rotational) -eq 0 ]] || return 1
}

die()
{
    # die peacefully
    local exitcode=0
    (( $# == 0 )) || exitcode=$1
    tput cnorm
    if [[ -d $MNT ]] && cd; then
        # use `fuser` to kill processes using the mounted directory before umounting it
        fuser -km $MNT
        umount_dir $MNT

        # when passed 127 as the exit code, kill the loop mount to avoid hangups and reboot the system
        if [[ $exitcode -eq 127 ]]; then
            fuser -km /run/archiso/bootmnt
            umount -l /run/archiso/bootmnt
            systemctl -i reboot
        fi
    fi
    exit $exitcode
}

sigint()
{
    # used to trap SIGINT and cleanly exit the program
    printf "\n** CTRL-C caught"
    die 1
}

print4()
{
    # takes an arbitrary number of input fields and prints them out in fourths on separate lines
    local str="$*"
    if [[ ${#str} -gt $(( ${COLUMNS:-$(tput cols)} / 2 )) ]]; then
        q=$(awk '{print int(NF / 4)}' <<< "$str")
        str="$(awk '{
            pkgs1=pkgs2=pkgs3=pkgs4=""
            for (i=1;i<'"$q"';i++) {
                if (i == 1) {
                    pkgs1=$i
                } else {
                    pkgs1=pkgs1" "$i
                }
            }
            for (i='"$q"';i<'"$((q * 2))"';i++) {
                pkgs2=pkgs2" "$i
            }
            for (i='"$((q * 2))"';i<'"$((q * 3))"';i++) {
                pkgs3=pkgs3" "$i
            }
            for (i='"$((q * 3))"';i<NF;i++) {
                pkgs4=pkgs4" "$i
            }
            printf "%s\n           %s\n           %s\n           %s", pkgs1, pkgs2, pkgs3, pkgs4
        }' <<< "$str")"
    fi
    printf "%s\n" "$str"
}

oneshot()
{
    [[ -e /tmp/.ai_$1 || ! $(type $1) ]] && return 0
    $1 || return 1
    touch "/tmp/.ai_$1"
    return 0
}

system_devices()
{
    IGNORE_DEV="$(lsblk -lno NAME,MOUNTPOINT | awk '/\/run\/archiso\/bootmnt/ {sub(/[1-9]/, ""); print $1}')"
    if [[ $IGNORE_DEV ]]; then
        SYS_DEVS="$(lsblk -lno NAME,SIZE,TYPE | awk '/disk/ && !'"/$IGNORE_DEV/"' {print "/dev/" $1 " " $2}')"
    else
        SYS_DEVS="$(lsblk -lno NAME,SIZE,TYPE | awk '/disk/ {print "/dev/" $1 " " $2}')"
    fi
    DEV_COUNT="$(wc -l <<< "$SYS_DEVS")"
    declare -grx SYS_DEVS IGNORE_DEV DEV_COUNT
}

system_identify()
{
    declare -g IS_64BIT=false
    local efidir="/sys/firmware/efi"

    if grep -q 'GenuineIntel' /proc/cpuinfo; then
        declare -gx UCODE="intel-ucode"
    elif grep -q 'AuthenticAMD' /proc/cpuinfo; then
        declare -gx UCODE="amd-ucode"
    fi

    if grep -qi 'apple' /sys/class/dmi/id/sys_vendor; then
        modprobe -r -q efivars
    else
        modprobe -q efivarfs
    fi

    if [[ -d $efidir ]]; then
        SYS="UEFI"
        [[ $(cat $efidir/fw_platform_size) == 64 ]] && export IS_64BIT=true
        grep -q $efidir/efivars <<< "$(mount)" || mount -t efivarfs efivarfs $efidir/efivars
    else
        SYS="BIOS"
    fi

    # shellcheck disable=2034
    declare -g BT="$DIST Installer - $SYS (x86_64) - Version $VER"
    return 0
}

system_checks()
{
    [[ $(whoami) == "root" ]] || { infobox "$_ErrTitle" "$_NotRoot\n$_Exit" && die 1; }
    grep -qw 'lm' /proc/cpuinfo || { infobox "$_ErrTitle" "$_Not64Bit\n$_Exit" && die 1; }

    if grep -qw 'BCM4352' <<< "$(lspci -vnn -d 14e4:)"; then
        rmmod bcma >/dev/null 2>&1
        rmmod wl >/dev/null 2>&1
        modprobe wl >/dev/null 2>&1
        export BROADCOM_WL=true
    fi

    if ! curl -s --head 'https://www.archlinux.org/mirrorlist/all/' | sed '1q' | grep -qw '200'; then
        if [[ $(systemctl is-active NetworkManager) == "active" ]] && hash nmtui >/dev/null 2>&1; then
            tput civis; nmtui-connect
            if ! curl -s --head 'https://www.archlinux.org/mirrorlist/all/' | sed '1q' | grep -qw '200'; then
                infobox "$_ErrTitle" "$_NoNetwork" 3
                die 1
            fi
        fi
    fi

    return 0
}

preinstall_checks()
{
    [[ $(lsblk -o MOUNTPOINT) =~ $MNT ]] || { msgbox "$_ErrTitle" "$_ErrNoMount"; export SELECTED=4; return 1; }
    if [[ $# -eq 1 ]]; then
        [[ $CONFIG_DONE == true ]] || { msgbox "$_ErrTitle" "$_ErrNoConfig"; export SELECTED=5; return 1; }
    fi
    return 0
}

echeck()
{
    # return if the last process exited normally
    local last_exit_code=$?
    (( last_exit_code == 0 )) && return 0
    local msg="\nThe command exited abnormally: $1"

    # get error message from logfile and strip any non-printable characters,
    # escape sequences, and other messy text
    local err
    err="$(sed 's/[^[:print:]]//g; s/\[[0-9\;:]*\?m//g; s/==> //g; s/] ERROR:/]\nERROR:/g' "$ERR")"
    [[ $err != "" ]] && msgbox "$_ErrTitle" "$msg\n\nWith the following error message:\n\n$err"

    msg="$([[ $err == "" ]] && printf "%s" "$msg")\n$_ErrChoice"
    if [[ -e $DBG && $TERM == 'linux' ]]; then
        msg="$([[ $err == "" ]] && printf "%s" "$msg")\n$_ErrChoiceConsole"
        yesno "$_ErrTitle" "$msg" "Exit & Open Log" "Ignore & Continue" && { more $DBG; die; }
    else
        yesno "$_ErrTitle" "$msg" "Exit" "Ignore & Continue" && die
    fi
    return 0
}

debug()
{
    set -x
    exec 3>| $DBG
    BASH_XTRACEFD=3
    if [[ $DISPLAY && $TERM != 'linux' ]]; then
        if hash 'termite' >/dev/null 2>&1; then
            termite -e "tail -f $DBG" &
        else
            xterm -e "tail -f $DBG" &
        fi
    fi
    export DEBUG=true
}

umount_dir()
{
    swapoff -a
    [[ -d $1 ]] && umount -R $1 >/dev/null 2>&1
    return 0
}
